import pytest


@pytest.mark.asyncio
def test_pin_add(client):
    yield from client.pin.add("ipfs-path", True)
    client.transport.request.assert_called_with(
        'get', endpoint='/pin/add',
        params=[('arg', 'ipfs-path'), ('recursive', True)]
    )


@pytest.mark.asyncio
def test_pin_ls(client):
    yield from client.pin.ls('ipfs-path', 'all', False)
    client.transport.request.assert_called_with(
        'get', endpoint='/pin/ls',
        params=[('arg', 'ipfs-path'), ('type', 'all'), ('quiet', False)]
    )


@pytest.mark.asyncio
def test_pin_rm(client):
    # curl "http://localhost:5001/api/v0/pin/rm?arg=<ipfs-path>&recursive=true"
    yield from client.pin.rm('ipfs-path', True)
    client.transport.request.assert_called_with(
        'get', endpoint='/pin/rm',
        params=[('arg', 'ipfs-path'), ('recursive', True)]
    )


class Tar(object):
    base_endpoint = '/tar'

    def __init__(self, transport):
        self.transport = transport

    def add(self, filestream):
        endpoint = self.base_endpoint + '/add'
        data = {
            'file': filestream
        }
        return (yield from self.transport.request('post', endpoint=endpoint, data=data))

    def cat(self, ipfs_path):
        endpoint = self.base_endpoint + '/cat'
        params = [('arg', ipfs_path)]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))



class Bitswap(object):
    """Bitswap agent."""

    base_endpoint = '/bitswap'

    def __init__(self, transport):
        self.transport = transport

    def stat(self):
        """Show some diagnostic information on the bitswap agent.

        :return: dict -- return
           ::

              {
                 "ProvideBufLen": "<int>",
                 "Wantlist": [
                    "<string>"
                 ],
                 "Peers": [
                    "<string>"
                 ],
                 "BlocksReceived": "<int>",
                 "DupBlksReceived": "<int>",
                 "DupDataReceived": "<uint64>"
               }

        """
        endpoint = self.base_endpoint + '/stat'
        res = yield from self.transport.request('get', endpoint=endpoint)
        return (yield from res.json())

    def unwant(self, key):
        """Remove a given block from your wantlist.

        :param key: Key(s) to remove from your wantlist.
        :type key: str

        :return: bytes -- file

        """
        params = {
            'arg': key
        }
        endpoint = self.base_endpoint + '/unwant'
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.read())

    def wantlist(self, peer: str):
        """Show blocks currently on the wantlist.

        :param peer: Specify which peer to show wantlist for

        :return: dict -- return
           ::

              {
                 "Keys": [
                    "<string>"
                 ]
              }
        """
        params = {
            'peer': peer
        }
        endpoint = self.base_endpoint + '/wantlist'
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.json())



class Pin(object):

    base_endpoint = '/pin'

    def __init__(self, transport):
        self.transport = transport

    def add(self, ipfs_path, recursive=True):
        endpoint = self.base_endpoint + '/add'
        params = [
            ('arg', ipfs_path),
            ('recursive', recursive)
        ]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))

    def ls(self, ipfs_path, _type='all', quiet=False):
        endpoint = self.base_endpoint + '/ls'
        params = [
            ('arg', ipfs_path),
            ('type', _type),
            ('quiet', quiet)
        ]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))

    def rm(self, ipfs_path, recursive=True):
        endpoint = self.base_endpoint + '/rm'
        params = [
            ('arg', ipfs_path),
            ('recursive', recursive)
        ]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))



class Name(object):

    base_endpoint = '/name'

    def __init__(self, transport):
        self.transport = transport

    def publish(self, ipfs_path, resolve=True, lifetime='24h', ttl=None):
        endpoint = self.base_endpoint + '/publish'
        params = [
            ('arg', ipfs_path),
            ('resolve', resolve),
            ('lifetime', lifetime),
            ('ttl', ttl)
        ]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))

    def resolve(self, ipns_name, recursive, nocache):
        endpoint = self.base_endpoint + '/resolve'
        params = [
            ('arg', ipns_name),
            ('recursive', recursive),
            ('nocache', nocache)
        ]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))



import pytest


@pytest.mark.asyncio
def test_log_level(client):
    yield from client.log.level("subsystem", "debug")
    client.transport.request.assert_called_with(
        'get', endpoint='/log/level', params=[('arg', 'subsystem'), ('arg', 'debug')]
    )


@pytest.mark.asyncio
def test_log_ls(client):
    yield from client.log.ls()
    client.transport.request.assert_called_with(
        'get', endpoint='/log/ls'
    )


@pytest.mark.asyncio
def test_log_tail(client):
    yield from client.log.tail()
    client.transport.request.assert_called_with(
        'get', endpoint='/log/tail'
    )



import pytest


@pytest.mark.asyncio
def test_diag_cmds_clear(client):
    yield from client.diag.cmds_clear()
    client.transport.request.assert_called_with('get', endpoint='/diag/cmds/clear')


@pytest.mark.asyncio
def test_diag_cmds_set_time(client):
    yield from client.diag.cmds_set_time("time")
    client.transport.request.assert_called_with('get', endpoint='/diag/cmds/set-time', params={
        'arg': 'time'
    })


@pytest.mark.asyncio
def test_diag_net(client):
    yield from client.diag.net("text")
    client.transport.request.assert_called_with('get', endpoint='/diag/net', params={
        'vis': 'text'
    })


@pytest.mark.asyncio
def test_diag_sys(client):
    yield from client.diag.sys()
    # curl "http://localhost:5001/api/v0/diag/sys"
    client.transport.request.assert_called_with('get', endpoint='/diag/sys')



class DHT(object):

    base_endpoint = '/dht'

    def __init__(self, transport):
        self.transport = transport

    def findpeer(self, peer_id, verbose=False):
        """Query the DHT for all of the multiaddresses associated with a Peer ID.

        :param peer_id: ID of the peer to search for
        :type peer_id: str

        :param verbose: Print extra information. Default: False
        :type verbose: bool

        :return: dict -- return
           ::

              {
                 "ID": "<string>",
                 "Type": "<int>",
                 "Responses": None,
                 "Extra": "<string>"
              }

        """
        endpoint = self.base_endpoint + '/findpeer'
        params = {
            'arg': peer_id,
            'verbose': verbose
        }
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.json())

    def findprovs(self, key, verbose=False):
        """Find peers in the DHT that can provide a specific value, given a key.

        :param key: key to find providers for
        :type key: str

        :param verbose: Print extra information. Default: False
        :type verbose: bool

        :return: dict -- return
           ::

              {
                 "ID": "<string>",
                 "Type": "<int>",
                 "Responses": null,
                 "Extra": "<string>"
              }

        """
        endpoint = self.base_endpoint + '/findprovs'
        params = {
            'arg': key,
            'verbose': verbose
        }
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.json())

    def get(self, key, verbose=False):
        """Given a key, query the DHT for its best value.

        :param key: key to find a value for
        :type key: str

        :param verbose: Print extra information
        :type verbose: bool

        :return: dict -- return
           ::

              {
                 "ID": "<string>",
                 "Type": "<int>",
                 "Responses": None,
                 "Extra": "<string>"
              }

        """
        endpoint = self.base_endpoint + '/get'
        params = {
            'arg': key,
            'verbose': verbose
        }
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.json())

    def put(self, key, value, verbose=False):
        """Write a key/value pair to the DHT.

        :param key: The key to store the value at
        :type key: str

        :param value: value to store
        :type value: str

        :param verbose: Print extra information. Default: False
        :type verbose: bool

        :return: dict -- return
           ::

              {
                 "ID": "<string>",
                 "Type": "<int>",
                 "Responses": None,
                 "Extra": "<string>"
              }
        """
        endpoint = self.base_endpoint + '/put'
        params = [('arg', key), ('arg', value), ('verbose', verbose)]
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.json())

    def query(self, peer_id, verbose=False):
        """Find the closest Peer IDs to a given Peer ID by querying the DHT.

        :param peer_id: The peerID to run the query against.
        :type peer_id: str

        :param verbose: Print extra information. Default: False
        :type verbose: bool

        :return: dict -- return
           ::

              {
                 "ID": "<string>",
                 "Type": "<int>",
                 "Responses": None,
                 "Extra": "<string>"
              }
        """
        endpoint = self.base_endpoint + '/query'
        params = {
            'arg': peer_id,
            'verbose': verbose
        }
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.json())



class Config(object):

    base_endpoint = '/config'

    def __init__(self, transport):
        self.transport = transport

    def replace(self, configfile):
        """Replaces the config with .

        :param configfile: file to use as the new config.

        :return: bytes -- file
        """
        endpoint = self.base_endpoint + '/replace'
        data = {
            'file': configfile
        }
        res = yield from self.transport.request('post', endpoint=endpoint, data=data)
        return (yield from res.read())

    def show(self):
        """Outputs the content of the config file.

        :return: bytes -- file config
        """
        endpoint = self.base_endpoint + '/show'
        res = yield from self.transport.request('get', endpoint=endpoint)
        return (yield from res.read())



class Bootstrap(object):

    base_endpoint = '/bootstrap'

    def __init__(self, transport):
        self.transport = transport

    def add_default(self):
        """Add default peers to the bootstrap list.

        :return: dict -- return
           ::

              {
                 "Peers": [
                    "<string>"
                 ]
              }

        """
        endpoint = self.base_endpoint + '/add/default'
        res = yield from self.transport.request('get', endpoint=endpoint)
        return (yield from res.json())

    def list(self):
        """Show peers in the bootstrap list.

        :return: dict -- return
           ::

              {
                 "Peers": [
                    "<string>"
                 ]
              }

        """
        endpoint = self.base_endpoint + '/list'
        res = yield from self.transport.request('get', endpoint=endpoint)
        return (yield from res.json())

    def rm_all(self):
        """Removes all peers from the bootstrap list.

        :return: dict -- return
           ::

              {
                 "Peers": [
                    "<string>"
                 ]
              }

        """
        endpoint = self.base_endpoint + '/rm/all'
        res = yield from self.transport.request('get', endpoint=endpoint)
        return (yield from res.json())



class Stats(object):

    base_endpoint = '/stats'

    def __init__(self, transport):
        self.transport = transport

    def bitswap(self):
        endpoint = self.base_endpoint + '/bitswap'
        return (yield from self.transport.request('get', endpoint=endpoint))

    def bw(self, peer=None, proto=None, poll=False, interval=None):
        endpoint = self.base_endpoint + '/bw'
        params = [
            ('peer', peer),
            ('proto', proto),
            ('poll', poll),
            ('interval', interval)
        ]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))

    def repo(self, human=False):
        endpoint = self.base_endpoint + '/repo'
        params = [('human', human)]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))



import pytest


@pytest.mark.asyncio
def test_tar_add(client, dummyfilestream):
    yield from client.tar.add(dummyfilestream)
    client.transport.request.assert_called_with(
        'post', endpoint='/tar/add', data={'file': dummyfilestream}
    )


@pytest.mark.asyncio
def test_tar_cat(client):
    yield from client.tar.cat('ipfs-path')
    client.transport.request.assert_called_with(
        'get', endpoint='/tar/cat', params=[('arg', 'ipfs-path')]
    )




import pytest


@pytest.mark.asyncio
def test_bitswap_wantlist(client):
    yield from client.bitswap.wantlist(peer="peer")
    client.transport.request.assert_called_with('get', endpoint='/bitswap/wantlist', params={
        "peer": "peer"
    })


@pytest.mark.asyncio
def test_bitswap_stat(client):
    yield from client.bitswap.stat()
    client.transport.request.assert_called_with('get', endpoint='/bitswap/stat')


@pytest.mark.asyncio
def test_bitswap_unwant(client):
    yield from client.bitswap.unwant("key")
    client.transport.request.assert_called_with('get', endpoint='/bitswap/unwant', params={
        "arg": "key"
    })



import pytest


@pytest.mark.asyncio
def test_file_ls(client):
    yield from client.file.ls("path")
    client.transport.request.assert_called_with('get', endpoint='/file/ls', params={"arg": "path"})



import pytest


@pytest.mark.asyncio
def test_name_publish(client):
    yield from client.name.publish('ipfs-path', True, "300s", "ttl")
    client.transport.request.assert_called_with(
        'get', endpoint='/name/publish',
        params=[
            ('arg', 'ipfs-path'),
            ('resolve', True),
            ('lifetime', '300s'),
            ('ttl', 'ttl')
        ]
    )


@pytest.mark.asyncio
def test_name_resolve(client):
    # curl "http://localhost:5001/api/v0/name/resolve?arg=<name>&recursive=false&nocache=false"
    yield from client.name.resolve('ipns-name', False, False)
    client.transport.request.assert_called_with(
        'get', endpoint='/name/resolve',
        params=[
            ('arg', 'ipns-name'),
            ('recursive', False),
            ('nocache', False)
        ]
    )



import pytest


@pytest.mark.asyncio
def test_resolve(client):
    yield from client.resolve("name", recursive=False)
    client.transport.request.assert_called_with('get', endpoint='/resolve', params={
        'arg': 'name',
        'recursive': False
    })


@pytest.mark.asyncio
def test_ping(client):
    yield from client.ping("123", count=10)
    client.transport.request.assert_called_with('get', endpoint='/ping', params={
        'arg': '123',
        'count': 10
    })


@pytest.mark.asyncio
def test_mount(client):
    yield from client.mount("ipfs-path", "ipns-path")
    client.transport.request.assert_called_with('get', endpoint='/mount', params={
        'ipfs-path': 'ipfs-path',
        'ipns-path': 'ipns-path'
    })


@pytest.mark.asyncio
def test_ls(client):
    yield from client.ls("ipfs-path", headers=False, resolve_type=True)
    client.transport.request.assert_called_with('get', endpoint="/ls", params={
        "arg": "ipfs-path",
        "headers": False,
        "resolve-type": True
    })


@pytest.mark.asyncio
def test_id(client):
    yield from client.id(peer_id="123", _format="")
    client.transport.request.assert_called_with('get', endpoint="/id", params={
        "arg": "123",
        "format": ""
    })


@pytest.mark.asyncio
def test_get(client):
    yield from client.get("ipfs-path", output="", archive=False, compress=False, compression_level=-1)
    client.transport.request.assert_called_with('get', endpoint="/get", params={
        "arg": "ipfs-path",
        "output": "",
        "archive": False,
        "compress": False,
        "compression-level": -1
    })


@pytest.mark.asyncio
def test_dns(client):
    yield from client.dns("domain-name", recursive=False)
    client.transport.request.assert_called_with(
        "get", endpoint="/dns",
        params={
            "arg": "domain-name",
            "recursive": False
        }
    )


@pytest.mark.asyncio
def test_cat(client):
    yield from client.cat("ipfs-path")
    client.transport.request.assert_called_with(
        "get", endpoint="/cat",
        params={
            "arg": "ipfs-path"
        }
    )


@pytest.mark.asyncio
def test_add(client, dummyfilestream):
    yield from client.add(
        filestream=dummyfilestream,
        recursive=False,
        quiet=False,
        silent=False,
        progress=False,
        trickle=False,
        only_hash=False,
        wrap_with_directory=False,
        hidden=False,
        chunker="",
        pin=True
    )
    client.transport.request.assert_called_with(
        "post", endpoint="/add",
        data={'file': dummyfilestream},
        params={
            "recursive": False,
            "quiet": False,
            "silent": False,
            "progress": False,
            "trickle": False,
            "only_hash": False,
            "wrap_with_directory": False,
            "hidden": False,
            "chunker": "",
            "pin": True
        }
    )


@pytest.mark.asyncio
def test_version(client):
    yield from client.version()
    client.transport.request.assert_called_with("get", endpoint="/version")



class File(object):
    base_endpoint = '/file'

    def __init__(self, transport):
        self.transport = transport

    def ls(self, path):
        """List directory contents for Unix filesystem objects.

        :param path: path to the IPFS object(s) to list links from
        :type path: str

        :return: dict -- return
           ::

              {
                 "Arguments": {
                    "<string>": "<string>"
                 },
                 "Objects": {
                    "<string>": {
                       "Hash": "<string>",
                       "Size": "<uint64>",
                       "Type": "<string>",
                       "Links": [
                          {
                             "Name": "<string>",
                             "Hash": "<string>",
                             "Size": "<uint64>",
                             "Type": "<string>"
                          }
                       ]
                    }
                 }
              }
        """
        endpoint = self.base_endpoint + '/ls'
        params = {
            "arg": path
        }
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.json())



class Patch(object):
    base_endpoint = '/object/patch'

    def __init__(self, transport):
        self.transport = transport

    def add_link(self, hashname, name, ipfsobject, create=False):
        """Add a link to a given object.

        :param hashname: The hash of the node to modify.
        :type hashname: str

        :param name: Name of link to create.
        :type name: str

        :param ipfsobject: IPFS object to add link to.
        :type ipfsobject: str

        :param create: Create intermediary nodes. Default: False
        :type create: bool

        :return: dict -- return
        """
        endpoint = self.base_endpoint + '/add-link'
        params = [
            ("arg", hashname),
            ("arg", name),
            ("arg", ipfsobject),
            ("create", create)
        ]
        res = yield from self.transport.request("get", endpoint=endpoint, params=params)
        return (yield from res.json())

    def append_data(self, filestream, node):
        """Append data to the data segment of a dag node.

        :param filestream: Data to append.
        :type filestream: io.Stream

        :param node: The hash of the node to modify.
        :type node: str

        :return: dict -- return
           ::

              {
                 "Hash": "<string>",
                 "Links": [
                    {
                       "Name": "<string>",
                       "Hash": "<string>",
                       "Size": "<uint64>"
                    }
                 ]
              }

        """
        endpoint = self.base_endpoint + '/append-data'
        params = [('arg', node)]
        data = {
            "file": filestream
        }
        res = yield from self.transport.request("post", endpoint=endpoint, params=params, data=data)
        return (yield from res.json())

    def rm_link(self, hashname, name):
        """Remove a link from an object.

        :param hashname:The hash of the node to modify.
        :type hashname: str

        :param name: Name of the link to remove.
        :type name: str

        :return: dict -- return
           ::

              {
                 "Hash": "<string>",
                 "Links": [
                    {
                       "Name": "<string>",
                       "Hash": "<string>",
                       "Size": "<uint64>"
                    }
                 ]
              }
        """
        endpoint = self.base_endpoint + '/rm-link'
        params = [
            ('arg', hashname),
            ('arg', name)
        ]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))

    def set_data(self, filestream, hashnode):
        endpoint = self.base_endpoint + '/set-data'
        params = [('arg', hashnode)]
        data = {
            "file": filestream
        }
        return (yield from self.transport.request('get', endpoint=endpoint, params=params, data=data))


class Object(object):
    base_endpoint = '/object'

    def __init__(self, transport):
        self.transport = transport
        self.patch = Patch(self.transport)

    def data(self, hash):
        endpoint = self.base_endpoint + '/data'
        params = {
            'arg': hash
        }
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))

    def diff(self, object1, object2, verbose=False):
        endpoint = self.base_endpoint + '/diff'
        params = [
            ('arg', object1),
            ('arg', object2),
            ('verbose', verbose)
        ]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))

    def get(self, key):
        endpoint = self.base_endpoint + '/get'
        params = {
            'arg': key
        }
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))

    def links(self, key, headers=False):
        endpoint = self.base_endpoint + '/links'
        params = {
            'arg': key,
            'headers': headers
        }
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))

    def new(self, template=None):
        endpoint = self.base_endpoint + '/new'
        params = {
            'arg': template
        }
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))

    def put(self, filestream, inputenc='json', datafieldenc='text'):
        endpoint = self.base_endpoint + '/put'
        data = {
            'file': filestream
        }
        params = {
            'inputenc': inputenc,
            'datafieldenc': datafieldenc
        }
        return (yield from self.transport.request('get', endpoint=endpoint, data=data, params=params))

    def stat(self, key):
        endpoint = self.base_endpoint + '/stat'
        params = [('arg', key)]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))




import pytest


@pytest.mark.asyncio
def test_repo_fsck(client):
    yield from client.repo.fsck()
    client.transport.request.assert_called_with(
        'get',
        endpoint='/repo/fsck'
    )


@pytest.mark.asyncio
def test_repo_gc(client):
    yield from client.repo.gc(False)
    client.transport.request.assert_called_with(
        'get',
        endpoint='/repo/gc',
        params=[('quiet', False)]
    )


@pytest.mark.asyncio
def test_repo_stat(client):
    yield from client.repo.stat(False)
    client.transport.request.assert_called_with(
        'get', endpoint='/repo/stat',
        params=[('human', False)]
    )


@pytest.mark.asyncio
def test_repo_verify(client):
    yield from client.repo.verify()
    client.transport.request.assert_called_with(
        'get', endpoint='/repo/verify'
    )


@pytest.mark.asyncio
def test_repo_version(client):
    # curl "http://localhost:5001/api/v0/repo/version?quiet=<value>"
    yield from client.repo.version(True)
    client.transport.request.assert_called_with(
        'get', endpoint='/repo/version',
        params=[('quiet', True)]
    )



class Repo(object):

    base_endpoint = '/repo'

    def __init__(self, transport):
        self.transport = transport

    def fsck(self):
        endpoint = self.base_endpoint + '/fsck'
        return (yield from self.transport.request('get', endpoint=endpoint))

    def gc(self, quiet=False):
        endpoint = self.base_endpoint + '/gc'
        params = [('quiet', quiet)]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))

    def stat(self, human=False):
        endpoint = self.base_endpoint + '/stat'
        params = [('human', human)]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))

    def verify(self):
        endpoint = self.base_endpoint + '/verify'
        return (yield from self.transport.request('get', endpoint=endpoint))

    def version(self, quiet=None):
        endpoint = self.base_endpoint + '/version'
        params = [('quiet', quiet)]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))



import pytest


@pytest.mark.asyncio
def test_dht_findpeer(client):
    yield from client.dht.findpeer('peer_id', verbose=False)
    client.transport.request.assert_called_with('get', endpoint='/dht/findpeer', params={
        'arg': 'peer_id',
        'verbose': False
    })


@pytest.mark.asyncio
def test_dht_findprovs(client):
    yield from client.dht.findprovs('key', verbose=False)
    client.transport.request.assert_called_with('get', endpoint='/dht/findprovs', params={
        'arg': 'key',
        'verbose': False
    })


@pytest.mark.asyncio
def test_dht_get(client):
    yield from client.dht.get('key', verbose=False)
    client.transport.request.assert_called_with('get', endpoint='/dht/get', params={
        'arg': 'key',
        'verbose': False
    })


@pytest.mark.asyncio
def test_dht_put(client):
    yield from client.dht.put('key', 'value', verbose=False)
    client.transport.request.assert_called_with('get', endpoint='/dht/put', params=[
        ('arg', 'key'), ('arg', 'value'), ('verbose', False)
    ])


@pytest.mark.asyncio
def test_dht_query(client):
    yield from client.dht.query("peer_id")
    client.transport.request.assert_called_with('get', endpoint='/dht/query', params={
        'arg': 'peer_id',
        'verbose': False
    })



class Files(object):

    base_endpoint = '/files'

    def __init__(self, transport):
        self.transport = transport

    def cp(self, source, dest):
        """Copy files into mfs.
        :param source: Source object to copy.
        :type source: str

        :param dest: Destination to copy object to.
        :type dest: str

        :return: bytes -- file
        """
        endpoint = self.base_endpoint + '/cp'
        params = [("arg", source), ("arg", dest)]
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.read())

    def flush(self, path="/"):
        """Flush a given path's data to disk.

        :param path: Path to flush. Default: '/'.
        :type path: str

        :return: bytes -- file
        """
        endpoint = self.base_endpoint + '/flush'
        params = {
            'arg': path
        }
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.read())

    def ls(self, path='/', long=False):
        """List directories.

        :param path: Path to show listing for. Defaults to '/'.
        :type path: str

        :param long: Use long listing format
        :type long: bool

        :return: dict -- return
           ::

              {
                 "Entries": [
                    {
                       "Name": "<string>",
                       "Type": "<int>",
                       "Size": "<int64>",
                       "Hash": "<string>"
                    }
                 ]
              }

        """
        endpoint = self.base_endpoint + '/ls'
        params = {
            'arg': path,
            'l': long
        }
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.json())

    def mkdir(self, path, parent=False):
        """Make directories.

        :param path: Path to dir to make
        :type path: str

        :param parent: No error if existing, make parent directories as needed.
        :type parent: bool

        :return: bytes -- file
        """
        endpoint = self.base_endpoint + '/mkdir'
        params = {
            'arg': path,
            'parent': parent
        }
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.read())

    def mv(self, source, dest):
        """Move files.

        :param source: Source file to move.
        :type source: str

        :param dest: Destination path for file to be moved to.
        :type dest: str

        :return: bytes -- file
        """
        endpoint = self.base_endpoint + '/mv'
        params = [('arg', source), ('arg', dest)]
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.read())

    def read(self, path, offset=None, count=None):
        """Read a file in a given mfs.

        :param path: Path to file to be read.
        :type path: str

        :param offset: Byte offset to begin reading from.
        :type offset: int

        :param count: Maximum number of bytes to read.
        :type count: int

        :return: bytes -- file
        """
        endpoint = self.base_endpoint + '/read'
        params = {
            'arg': path,
            'offset': offset,
            'count': count
        }
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.read())

    def rm(self, path, recursive=False):
        """Remove a file.

        :param path: File to remove.
        :type path: str

        :param recursive: Recursively remove directories.
        :type recursive: bool

        :return: bytes -- file
        """
        endpoint = self.base_endpoint + '/rm'
        params = {
            'arg': path,
            'recursive': recursive
        }
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.read())

    def stat(self, path, _hash=False, size=False):
        """Display file status.

        :param path: Path to node to stat.
        :type path: str

        :param _hash: Print only hash. Implies '--format='.
        :type _hash: bool

        :param size: Print only size.
        :type size: bool

        :return: dict -- return
           ::

              {
                 "Hash": "<string>",
                 "Size": "<uint64>",
                 "CumulativeSize": "<uint64>",
                 "Blocks": "<int>",
                 "Type": "<string>"
              }

        """
        endpoint = self.base_endpoint + '/stat'
        params = {
            'arg': path,
            'hash': _hash,
            'size': size
        }
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.json())

    def write(self, path, filestream, offset=None, create=False, truncate=False, count=None):
        """Write to a mutable file in a given filesystem.

        :param path: Path to write to.
        :type path: str

        :param filestream: Data to write.
        :type filestream: io.BaseStream

        :param offset: Byte offset to begin writing at.
        :type offset: int

        :param create: Create the file if it does not exist.
        :type create: bool

        :param truncate: Truncate the file to size zero before writing.
        :type truncate: bool

        :param count: Maximum number of bytes to read.
        :type count: int

        :return: bytes -- file
        """
        endpoint = self.base_endpoint + '/write'
        params = {
            'arg': path,
            'offset': offset,
            'create': create,
            'truncate': truncate,
            'count': count
        }
        data = {
            'file': filestream
        }
        res = yield from self.transport.request('post', endpoint=endpoint, data=data, params=params)
        return (yield from res.read())



import pytest


@pytest.mark.asyncio
def test_swarm_addrs_local(client):
    yield from client.swarm.addrs.local(False)
    client.transport.request.assert_called_with(
        'get', endpoint='/swarm/addrs/local',
        params=[('id', False)]
    )


@pytest.mark.asyncio
def test_swarm_connect(client):
    yield from client.swarm.connect('address')
    client.transport.request.assert_called_with(
        'get', endpoint='/swarm/connect',
        params=[('arg', 'address')]
    )


@pytest.mark.asyncio
def test_swarm_disconnect(client):
    yield from client.swarm.disconnect('address')
    client.transport.request.assert_called_with(
        'get', endpoint='/swarm/disconnect',
        params=[('arg', 'address')]
    )


@pytest.mark.asyncio
def test_swarm_filters_add(client):
    yield from client.swarm.filters.add('address')
    client.transport.request.assert_called_with(
        'get', endpoint='/swarm/filters/add',
        params=[('arg', 'address')]
    )


@pytest.mark.asyncio
def test_swarm_filters_rm(client):
    yield from client.swarm.filters.rm('address')
    client.transport.request.assert_called_with(
        'get', endpoint='/swarm/filters/rm',
        params=[('arg', 'address')]
    )


@pytest.mark.asyncio
def test_swarm_peers(client):
    yield from client.swarm.peers(None)
    client.transport.request.assert_called_with(
        'get', endpoint='/swarm/peers',
        params=[('verbose', None)]
    )



import pytest


@pytest.mark.asyncio
def test_refs_local(client):
    # curl "http://localhost:5001/api/v0/refs/local"
    yield from client.refs.local()
    client.transport.request.assert_called_with(
        'get',
        endpoint='/refs/local'
    )



class Addrs(object):
    base_endpoint = '/swarm/addrs'

    def __init__(self, transport):
        self.transport = transport

    def local(self, show_peer_id=False):
        endpoint = self.base_endpoint + '/local'
        params = [('id', show_peer_id)]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))


class Filters(object):
    base_endpoint = '/swarm/filters'

    def __init__(self, transport):
        self.transport = transport

    def add(self, addrs):
        endpoint = self.base_endpoint + '/add'
        params = [('arg', addrs)]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))

    def rm(self, addrs):
        endpoint = self.base_endpoint + '/rm'
        params = [('arg', addrs)]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))


class Swarm(object):
    base_endpoint = '/swarm'

    def __init__(self, transport):
        self.transport = transport

        self.addrs = Addrs(self.transport)

        self.filters = Filters(self.transport)

    def connect(self, addrs):
        endpoint = self.base_endpoint + '/connect'
        params = [('arg', addrs)]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))

    def disconnect(self, addrs):
        endpoint = self.base_endpoint + '/disconnect'
        params = [('arg', addrs)]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))

    def peers(self, verbose=None):
        endpoint = self.base_endpoint + '/peers'
        params = [('verbose', verbose)]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))



import pytest


@pytest.mark.asyncio
def test_stats_bitswap(client):
    yield from client.stats.bitswap()
    client.transport.request.assert_called_with(
        'get', endpoint='/stats/bitswap'
    )


@pytest.mark.asyncio
def test_stats_bw(client):
    yield from client.stats.bw('peer', 'proto', False, '1s')
    client.transport.request.assert_called_with(
        'get', endpoint='/stats/bw',
        params=[('peer', 'peer'), ('proto', 'proto'), ('poll', False), ('interval', '1s')]
    )


@pytest.mark.asyncio
def test_stats_repo(client):
    # curl "http://localhost:5001/api/v0/stats/repo?human=false"
    yield from client.stats.repo(False)
    client.transport.request.assert_called_with(
        'get', endpoint='/stats/repo', params=[('human', False)]
    )



import asyncio

from aipfs.transport import AsyncHTTP
from .bitswap import Bitswap
from .block import Block
from .bootstrap import Bootstrap
from .config import Config
from .dht import DHT
from .diag import Diag
from .file import File
from .files import Files
from .ipfsobject import Object as _Object
from .log import Log
from .name import Name
from .pin import Pin
from .refs import Refs
from .repo import Repo
from .stats import Stats
from .swarm import Swarm
from .tar import Tar


class IPFS(object):

    """IPFS api"""

    def __init__(self, loop=None, endpoint="http://localhost:5001/api/v0", transport=AsyncHTTP):

        # get event loop
        self.loop = loop or asyncio.get_event_loop()

        # instance transport
        self.transport = transport(loop=self.loop, base_url=endpoint)

        self.name = Name(self.transport)

        self.pin = Pin(self.transport)

        self.refs = Refs(self.transport)

        self.repo = Repo(self.transport)

        self.stats = Stats(self.transport)

        self.swarm = Swarm(self.transport)

        self.tar = Tar(self.transport)

        self.bitswap = Bitswap(self.transport)

        self.block = Block(self.transport)

        self.bootstrap = Bootstrap(self.transport)

        self.config = Config(self.transport)

        self.dht = DHT(self.transport)

        self.diag = Diag(self.transport)

        self.file = File(self.transport)

        self.files = Files(self.transport)

        self.log = Log(self.transport)

        self.object = _Object(self.transport)

    @asyncio.coroutine
    def add(self, filestream, **kwargs):
        """Add a file to ipfs

        :param filestream: Stream file to be added to IPFS.
        :type filestream: io.IOBase

        :return:  dict -- return code.
           ::

              {
                 "Name": "<string>",
                 "Hash": "<string>",
                 "Bytes": "<int64>"
              }
        """
        data = {
            "file": filestream
        }

        params = {
            "recursive": kwargs.get("recursive", False),
            "quiet": kwargs.get("quiet", False),
            "silent": kwargs.get("silent", False),
            "progress": kwargs.get("progrees", False),
            "trickle": kwargs.get("trickle", False),
            "only_hash": kwargs.get("only_hash", False),
            "wrap_with_directory": kwargs.get("wrap_with_directory", False),
            "hidden": kwargs.get("hidden", False),
            "chunker": kwargs.get("chunker", None),
            "pin": kwargs.get("pin", True)
        }

        res = yield from self.transport.request('post', endpoint="/add", data=data, params=params)
        return (yield from res.json())

    @asyncio.coroutine
    def cat(self, path):
        """Show IPFS object data

        :param path: ipfs path object
        :type path: str

        :return: bytes -- file
        """
        res = yield from self.transport.request('get', endpoint="/cat", params={"arg": path})
        return (yield from res.read())

    @asyncio.coroutine
    def dns(self, domain_name, recursive=False):
        """DNS link resolver

        :param domain_name: domain-name name to resolve
        :type domain_name: str

        :param recursive: Resolve until the result is not a DNS link. Default: False
        :type recursive: bool

        :return: dict -- return.
           ::

              {
                 "Path": "<string>"
              }
        """
        params = {
            "arg": domain_name,
            "recursive": recursive
        }
        res = yield from self.transport.request('get', endpoint="/dns", params=params)
        return (yield from res.json)

    @asyncio.coroutine
    def get(self, path, **kwargs):
        """Download IPFS objects.

        :param path: path IPFS object
        :type path: str

        :return: bytes -- file body
        """
        params = {
            "arg": path,
            "output": kwargs.get("output", None),
            "archive": kwargs.get("archive", False),
            "compress": kwargs.get("compress", False),
            "compression-level": kwargs.get("compression_level", -1)
        }
        res = yield from self.transport.request('get', endpoint="/get", params=params)
        return (yield from res.read())

    @asyncio.coroutine
    def id(self, peer_id="", _format=""):
        """Show IPFS Node ID info.

        :param peer_id: Peer.ID of node to look up.
        :type peer_id: str

        :param _format: Optional output format
        :type _format: str

        :return: dict -- return
           ::

              {
                 "ID": "<string>",
                 "PublicKey": "<string>",
                 "Addresses": [
                    "<string>"
                 ],
                 "AgentVersion": "<string>",
                 "ProtocolVersion": "<string>"
              }
        """
        params = {
            "arg": peer_id,
            "format": _format
        }
        res = yield from self.transport.request('get', endpoint="/id", params=params)
        return (yield from res.json())

    @asyncio.coroutine
    def ls(self, path, headers=False, resolve_type=True):
        """List links from an objects.

        :param path: The path to the IPFS object(s) to list links from
        :type path: str

        :param headers: Print table headers (Hash, Size, Name). Default: False
        :type headers: bool

        :param resolve_type: Resolve linked objects to find out their types. Default: True
        :type resolve_type: bool

        :return: dict -- return
           ::

              {
                 "Objects": [
                    {
                       "Hash": "<string>",
                       "Links": [
                          {
                             "Name": "<string>",
                             "Hash": "<string>",
                             "Size": "<uint64>",
                             "Type": "<int32>"
                          }
                       ]
                    }
                 ]
              }
        """
        params = {
            "arg": path,
            "headers": headers,
            "resolve-type": resolve_type
        }
        res = yield from self.transport.request('get', endpoint="/ls", params=params)
        return (yield from res.json())

    @asyncio.coroutine
    def mount(self, ipfs_path=None, ipns_path=None):
        """Mounts IPFS to the filesystem (read-only).

        :param ipfs_path: The path where IPFS should be mounted.
        :type ipfs_path: str

        :param ipns_path: The path where IPNS should be mounted.
        :type ipns_path: str

        :return: dict -- return code
           ::

              {
                 "IPFS": "<string>",
                 "IPNS": "<string>",
                 "FuseAllowOther": "<bool>"
              }
        """
        params = {
            "ipfs-path": ipfs_path,
            "ipns-path": ipns_path
        }
        res = yield from self.transport.request('get', endpoint="/mount", params=params)
        return (yield from res.json())

    @asyncio.coroutine
    def ping(self, peer_id, count=10):
        """Send echo request packets to IPFS hosts.

        :param peer_id: ID of peer to be pinged
        :type peer_id: str

        :param count: Number of ping messages to send. Default: 10
        :type count: int

        :return: dict -- return
           ::

              {
                 "Success": "<bool>",
                 "Time": "<int64>",
                 "Text": "<string>"
              }
        """
        params = {
            'arg': peer_id,
            'count': count
        }
        res = yield from self.transport.request('get', endpoint='/ping', params=params)
        return (yield from res.json())

    @asyncio.coroutine
    def resolve(self, ipfsname, recursive=False):
        """Resolve the value of names to IPFS.

        :param ipfsname: The name to resolve.
        :type ipfsname: str

        :param recursive: Resolve until the result is an IPFS name. Default: False
        :type recursive: bool

        :return: dict -- return
           ::

              {
                 "Path": "<string>"
              }
        """
        params = {
            'arg': ipfsname,
            'recursive': recursive
        }
        res = yield from self.transport.request('get', endpoint='/resolve', params=params)
        return (yield from res.json())

    @asyncio.coroutine
    def version(self):
        """Shows ipfs version information.

        :return: dict -- return
           ::

              {
                 "Version": "<string>",
                 "Commit": "<string>",
                 "Repo": "<string>",
                 "System": "<string>",
                 "Golang": "<string>"
              }
        """
        return (yield from self.transport.request('get', endpoint="/version"))

    def close(self):
        """Close all transport"""
        self.transport.close()

    def __enter__(self):
        return self

    def __exit__(self):
        self.close()



class Diag(object):

    base_endpoint = '/diag'

    def __init__(self, transport):
        self.transport = transport

    def cmds_clear(self):
        """Clear inactive requests from the log.

        :return: bytes -- file
        """
        endpoint = self.base_endpoint + '/cmds/clear'
        res = yield from self.transport.request('get', endpoint=endpoint)
        return (yield from res.read())

    def cmds_set_time(self, time):
        """Set how long to keep inactive requests in the log.

        :param time: Time to keep inactive requests in log.
        :type time: str

        :return: bytes -- file
        """
        endpoint = self.base_endpoint + '/cmds/set-time'
        params = {
            'arg': time
        }
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.read())

    def net(self, vis="text"):
        """Generates a network diagnostics report.

        :param vis: Output format. One of: d3, dot, text. Default: "text".
        :type vis: str

        :return: bytes -- file
        """
        endpoint = self.base_endpoint + '/net'
        params = {
            'vis': vis
        }
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.read())

    def sys(self):
        """Prints out system diagnostic information.

        :return: bytes -- file
        """
        endpoint = self.base_endpoint + '/sys'
        res = yield from self.transport.request('get', endpoint=endpoint)
        return (yield from res.read())



import pytest


@pytest.mark.asyncio
def test_object_data(client):
    yield from client.object.data('hash')
    client.transport.request.assert_called_with('get', endpoint='/object/data', params={'arg': 'hash'})


@pytest.mark.asyncio
def test_object_diff(client):
    yield from client.object.diff("object1", "object2", False)
    client.transport.request.assert_called_with('get', endpoint='/object/diff', params=[
        ('arg', 'object1'), ('arg', 'object2'), ('verbose', False)
    ])


@pytest.mark.asyncio
def test_object_get(client):
    yield from client.object.get('key')
    client.transport.request.assert_called_with('get', endpoint='/object/get', params={'arg': 'key'})


@pytest.mark.asyncio
def test_object_links(client):
    yield from client.object.links('key', False)
    client.transport.request.assert_called_with('get', endpoint='/object/links',
                                                params={'arg': 'key', 'headers': False})


@pytest.mark.asyncio
def test_object_new(client):
    yield from client.object.new('template')
    client.transport.request.assert_called_with('get', endpoint='/object/new', params={'arg': 'template'})


@pytest.mark.asyncio
def test_object_put(client, dummyfilestream):
    yield from client.object.put(dummyfilestream, inputenc='json', datafieldenc='text')
    client.transport.request.assert_called_with(
        'get', endpoint='/object/put',
        data={
            'file': dummyfilestream
        }, params={
            'inputenc': 'json', 'datafieldenc': 'text'
        })


@pytest.mark.asyncio
def test_object_patch_add_link(client):
    yield from client.object.patch.add_link("hash", "namelink", "ipfsobject", False)
    client.transport.request.assert_called_with(
        'get', endpoint='/object/patch/add-link', params=[
            ("arg", "hash"),
            ("arg", "namelink"),
            ("arg", "ipfsobject"),
            ("create", False)
        ]
    )


@pytest.mark.asyncio
def test_object_patch_append_data(client, dummyfilestream):
    yield from client.object.patch.append_data(dummyfilestream, "node")
    client.transport.request.assert_called_with(
        'post', endpoint='/object/patch/append-data', params=[
            ("arg",  "node")
        ], data={
            "file": dummyfilestream
        }
    )


@pytest.mark.asyncio
def test_object_patch_rm_link(client):
    yield from client.object.patch.rm_link("hashname", "name")
    client.transport.request.assert_called_with(
        'get', endpoint='/object/patch/rm-link', params=[
            ('arg', "hashname"),
            ('arg', "name")
        ]
    )


@pytest.mark.asyncio
def test_object_patch_set_data(client, dummyfilestream):
    yield from client.object.patch.set_data(dummyfilestream, "hashnode")
    client.transport.request.assert_called_with(
        'get', endpoint='/object/patch/set-data',
        params=[('arg', 'hashnode')], data={"file": dummyfilestream}
    )


@pytest.mark.asyncio
def test_object_stat(client):
    # curl "http://localhost:5001/api/v0/object/stat?arg=<key>"
    yield from client.object.stat("key")
    client.transport.request.assert_called_with(
        'get', endpoint='/object/stat', params=[('arg', "key")]
    )



class Refs(object):
    base_endpoint = '/refs'

    def __init__(self, transport):
        self.transport = transport

    def local(self):
        endpoint = self.base_endpoint + '/local'
        return (yield from self.transport.request('get', endpoint=endpoint))



import pytest


@pytest.mark.asyncio
def test_bootstrap_add_default(client):
    yield from client.bootstrap.add_default()
    client.transport.request.assert_called_with('get', endpoint='/bootstrap/add/default')


@pytest.mark.asyncio
def test_bootstrap_list(client):
    yield from client.bootstrap.list()
    client.transport.request.assert_called_with('get', endpoint='/bootstrap/list')


@pytest.mark.asyncio
def test_bootstrap_rm_all(client):
    yield from client.bootstrap.rm_all()
    client.transport.request.assert_called_with('get', endpoint='/bootstrap/rm/all')



import pytest


@pytest.mark.asyncio
def test_config_edit(client):
    yield from client.config.edit()
    client.transport.request.assert_called_with('get', endpoint='/config/edit')


@pytest.mark.asyncio
def test_config_replace(client, dummyfilestream):
    yield from client.config.replace(dummyfilestream)
    client.transport.request.assert_called_with('post', endpoint='/config/replace', data={
        'file': dummyfilestream
    })


@pytest.mark.asyncio
def test_config_show(client):
    yield from client.config.show()
    client.transport.request.assert_called_with('get', endpoint='/config/show')



import pytest


@pytest.mark.asyncio
def test_block_get(client):
    yield from client.block.get("hash")
    client.transport.request.assert_called_with("get", endpoint="/block/get", params={
        "arg": "hash"
    })


@pytest.mark.asyncio
def test_block_put(client, dummyfilestream):
    yield from client.block.put(dummyfilestream)
    client.transport.request.assert_called_with("post", endpoint="/block/put", data={
        'file': dummyfilestream
    })


@pytest.mark.asyncio
def test_block_stat(client):
    yield from client.block.stat("hash")
    client.transport.request.assert_called_with('get', endpoint="/block/stat", params={
        'arg': "hash"
    })


import pytest
from aipfs.transport import AsyncHTTP


@pytest.mark.asyncio
def test_async_http():
    with AsyncHTTP(base_url="https://python.org") as transport:
        res = yield from transport.request("get", "/")
        assert res.status == 200



import pytest


@pytest.mark.asyncio
def test_files_cp(client):
    yield from client.files.cp("source", "path")
    client.transport.request.assert_called_with(
        'get', endpoint='/files/cp', params=[('arg', 'source'), ('arg', 'path')]
    )


@pytest.mark.asyncio
def test_files_flush(client):
    yield from client.files.flush("path")
    client.transport.request.assert_called_with('get', endpoint='/files/flush', params={'arg': 'path'})


@pytest.mark.asyncio
def test_files_ls(client):
    yield from client.files.ls("path", False)
    client.transport.request.assert_called_with('get', endpoint='/files/ls', params={'arg': "path", "l": False})


@pytest.mark.asyncio
def test_files_mkdir(client):
    yield from client.files.mkdir("path", False)
    client.transport.request.assert_called_with('get', endpoint='/files/mkdir',
                                                params={'arg': 'path', 'parent': False})


@pytest.mark.asyncio
def test_files_mv(client):
    yield from client.files.mv("source", "dest")
    client.transport.request.assert_called_with('get', endpoint='/files/mv',
                                                params=[('arg', 'source'), ('arg', 'dest')])


@pytest.mark.asyncio
def test_files_read(client):
    yield from client.files.read("path", 1, 10)
    client.transport.request.assert_called_with('get', endpoint='/files/read', params={
        'arg': 'path',
        'offset': 1,
        'count': 10
    })


@pytest.mark.asyncio
def test_files_rm(client):
    yield from client.files.rm('path', False)
    client.transport.request.assert_called_with('get', endpoint='/files/rm', params={
        'arg': 'path',
        'recursive': False
    })


@pytest.mark.asyncio
def test_files_stat(client):
    yield from client.files.stat('path', False, False)
    client.transport.request.assert_called_with('get', endpoint='/files/stat', params={
        'arg': 'path',
        'hash': False,
        'size': False
    })


@pytest.mark.asyncio
def test_files_write(client, dummyfilestream):
    yield from client.files.write("path", dummyfilestream, 1, False, False, 10)
    client.transport.request.assert_called_with('post', endpoint='/files/write', params={
        'arg': "path",
        'offset': 1,
        'create': False,
        'truncate': False,
        'count': 10
    }, data={
        'file': dummyfilestream
    })
    pass



import os
import aipfs
import pytest
from asynctest import CoroutineMock


dir_path = os.path.dirname(os.path.realpath(__file__))


@pytest.fixture()
def client(request):
    mock = CoroutineMock
    mock.request = CoroutineMock()
    ipfs = aipfs.IPFS(transport=mock).__enter__()

    def teardown():
        ipfs.__exit__()

    request.addfinalizer(teardown)

    return ipfs


@pytest.fixture()
def dummyfilestream(request):
    file = open(dir_path + "/dummy_file")

    def teardown():
        file.close()

    request.addfinalizer(teardown)

    return file



#!/usr/bin/env python
from setuptools import setup, find_packages

setup(
    name='aipfs',

    version='0.0.1',

    description='IPFS daemon asyncio client',

    author='faisalburhanufin',

    author_email='faisalburhanudin@hotmail.net',

    url='https://github.com/faisalburhanudin/aipfs',

    download_url='https://github.com/faisalburhanudin/aipfs/archive/0.0.1.tar.gz',

    install_requires=[
        "aiohttp >= 1.2.0"
    ],

    packages=find_packages(),
)



class Block(object):
    """IPFS block"""

    base_endpoint = '/block'

    def __init__(self, transport):
        self.transport = transport

    def get(self, multihash: str):
        """Get a raw IPFS block.

        :param multihash: The base58 multihash of an existing block to get.

        :return: bytes -- file
        """
        endpoint = self.base_endpoint + '/get'
        params = {
            'arg': multihash
        }
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.read())

    def put(self, filestream):
        """Stores input as an IPFS block.

        :param filestream: The data to be stored as an IPFS block.

        :return: dict -- return
           ::

              {
                 "Key": "<string>",
                 "Size": "<int>"
              }

        """
        endpoint = self.base_endpoint + '/put'
        data = {
            "file": filestream
        }
        res = yield from self.transport.request('post', endpoint=endpoint, data=data)
        return (yield from res.json())

    def stat(self, multihash):
        """Print information of a raw IPFS block.

        :param multihash: The base58 multihash of an existing block to stat

        :return: dict -- return
           ::

             {
                "Key": "<string>",
                "Size": "<int>"
             }

        """
        endpoint = self.base_endpoint + '/stat'
        params = {
            'arg': multihash
        }
        res = yield from self.transport.request('get', endpoint=endpoint, params=params)
        return (yield from res.json())



import asyncio
import aiohttp


class AsyncHTTP(object):

    def __init__(self, loop=None, base_url=""):
        self.loop = loop or asyncio.get_event_loop()
        self.responses = {}
        self.client = aiohttp.ClientSession
        self.base_url = base_url
        self.sess = None

    @asyncio.coroutine
    def request(self, method, endpoint, **kwargs):
        """Requests to ipfs daemon

        """
        if not self.sess:
            self.sess = self.client(loop=self.loop)

        url = self.endpoint(endpoint)
        self.responses[url] = yield from self.sess.request(method=method, url=url, **kwargs)
        return self.responses[url]

    def endpoint(self, endpoint) -> str:
        """Get proper url"""
        return self.base_url + endpoint if self.base_url else endpoint

    def __enter__(self):
        """Enter context manager"""
        return self

    def __exit__(self, *args):
        """Close context manager"""
        self.close()

    def close(self):
        """Close session and all requests bucket"""
        if self.sess:
            self.sess.close()
        for v in self.responses.values():
            v.close()



class Log(object):

    base_endpoint = '/log'

    def __init__(self, transport):
        self.transport = transport

    def level(self, subsystem, level):
        endpoint = self.base_endpoint + '/level'
        params = [
            ('arg', subsystem),
            ('arg', level)
        ]
        return (yield from self.transport.request('get', endpoint=endpoint, params=params))

    def ls(self):
        endpoint = self.base_endpoint + '/ls'
        return (yield from self.transport.request('get', endpoint=endpoint))

    def tail(self):
        endpoint = self.base_endpoint + '/tail'
        return (yield from self.transport.request('get', endpoint=endpoint))
